 --[[
 Author:      Gilberto Camara
 Model name:  Daisyworld
 Date:        26.11.2014
 
]]--

--[[

Wood, A. J., G. J. Ackland, J. G. Dyke, H. T. P. Williams, and T. M. Lenton (2008), 
Daisyworld: A review, Rev. Geophys., 46.

Daisyworld consists of two different types of ‘‘daisy,’’ which may be considered distinct species (because there is no possibility of mixed replication of the types) or, alternatively, as distinct phenotypes of the same species. The two types are identified as either black or white according to their reflectivity or albedo.

--]]


tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;"

local sysdyn = require ("sysdyn")

--[[
Daisy growth rate function is a function of temperature. Maximum growth 
occurs at 22.5 degrees Celsius, and daisies grow from a minimum of 5 C to 
a maximum of 40 C.
--]]

---------------------------------------------------------------------------------------
function daisyGrowthRate (tempK) -- daisyGrowthRate as function of temperature
    local gr = 0.0
    local temp = tempK - 273
    if temp > 5.0 and temp < 40.0 then 
        gr = 1 - 0.003265 * (22.5 - temp)^2
    end      
    return gr
end
--------------------------------------------------------------------------------------

--[[ 
Calculates the average planet temperature using Boltzmann's law 

"The amount of thermal radiation emitted increases rapidly and the principal frequency of the radiation becomes higher with increasing temperatures" -- 

TempK^4 = average solar flux * luminosity (1 - albedo)

 Watson and Lovelock used 3668 watts per square meter as the solar flux,
 which is much higher than the average solar flux incident on the Earth today, 
 because the Earth has a relatively low planetary albedo and 
 a significant greenhouse effect. 
]]

SIGMA = 5.67E-08    -- Joules / per sec * square meters * TempK^^4 -- Stephen Boltzmann constant
SOLAR_FLUX = 3668   -- Watts/square meter

function calcPlanetTemp (lum, albedo)

    local tempK = (lum * SOLAR_FLUX * ( 1 - albedo)/(4*SIGMA)) ^0.25
    return tempK 
end
----------------------------------------------------------------

--[[
Calculates the temperatures near the daisies

The local temperatures Tw and Tb are defined by making a simplifying assumption about 
the heat transfer: a linearization of a diffusion term. 
A parameter q is defined as the heat transfer coefficient, 
thereby defining the local temperatures as

TempK^4 (daisy) = q (planet_albedo - daisy_albedo) + tempK^4 (planet)
--]]

HEAT_TRANSFER_COEF = 2.06E09  -- Kelvins^4  -- used by Watson and Lovelock

function tempNearDaisy (planet_temp, planet_albedo, daisy_albedo)
    local   temp_daisy  = (HEAT_TRANSFER_COEF * (planet_albedo - daisy_albedo) + planet_temp^4 ) ^0.25
    return  temp_daisy
end


 --[[
   Daisyworld model: implemented as a system dynamics model
   (uses sysdyn model - please see sysdyn.lua)
--]]
                
model = sysdyn.Model {
    
        -- sun luminosity (this is the main variable of the model)
        -- values beteween 0.70 and 1.6  support life in Daisyworld
        sun_luminosity         =   1.0, 
    
        -- variables 
        -- default values are approximate equilibrium values 
        -- when sun luminosity is equal to 1.0
        -- hint: change the starting conditions when changing the sun luminosity
        -- increase the white area for a strong sun
        -- increase the black area for a faint sun
        white_area            =   0.40, 
        black_area            =   0.273, 
        empty_area            =   0.327,	
        planet_albedo         =   0.532,
        planet_temp           =   295.0, -- in K (corresponds to 22 C) 
	    	                                                             
	    -- params (fixed assumptions)
        white_albedo          =   0.75,
        black_albedo          =   0.25, 
        soil_albedo           =   0.50, 
        decay_rate            =   0.3,
        
        -- control integration and display 
        delta_time  = 1,  -- integration step  -- 1.0, 0.5, 0.25, 0.125, 0.0625, 0.03125, 0.015625
        update_time = 1,  -- visualisation step
            
        changes = function (model)
            
            -- The model has two parts 
            
            -- First part of the model: Calculate the heat flow
           
            -- calculate the planetary albedo            
            model.planet_albedo = 
                    model.empty_area * model.soil_albedo  +   -- ground
                    model.white_area * model.white_albedo +   -- white daisy
                    model.black_area * model.black_albedo     -- black daisy
            -- calculate the planet temp
            model.planet_temp = calcPlanetTemp (model.sun_luminosity,
                                                model.planet_albedo)                           
            
            -- Second part of the model: 
            -- Calculate the growth of the daisies 
            -- generation growth occurs after the planet temperature has stabilized
            
            -- local temperatures near the white and black daisies                                
            local temp_near_white = tempNearDaisy (model.planet_temp, 
                    model.planet_albedo, model.white_albedo)
            local temp_near_black = tempNearDaisy (model.planet_temp, 
                    model.planet_albedo, model.black_albedo)
                
            -- potential growth rate of  daisies
            growth_rate_white  = daisyGrowthRate (temp_near_white)
            growth_rate_black  = daisyGrowthRate (temp_near_black)
                  
            -- actual growth of the white daisies         
            model.white_area = model.white_area 
                + (growth_rate_white*model.white_area*model.empty_area 
                - model.decay_rate*model.white_area) 

            -- actual growth of black daisies
            model.black_area =  model.black_area 
                + (growth_rate_black*model.black_area*model.empty_area 
                - model.decay_rate*model.black_area)
            
            -- resulting empty area
            model.daisy_area = model.black_area + model.white_area
            model.empty_area = 1.0 - model.daisy_area
                    
        end,
        
        graphics = { timeseries = { {"black_area", "white_area", "empty_area"},
                                 {"planet_albedo"}, {"sun_luminosity"},
                                 {"planet_temp"}}
                }
}
---------------------------------------------------------------------------------------

model:run (50)